﻿/*
    Fusion PDF Extractor v1.0 - Extract images and pages from PDF documents as JPG files
    Copyright (C) 2011  Fusion242 Ltd

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Threading;

using System.Drawing.Imaging;
using System.Diagnostics;

using iTextSharp.text;
using iTextSharp.text.pdf;
using System.IO;
using System.Runtime.InteropServices;

namespace FusionPDFExtractor
{
    public partial class Form1 : Form
    {
        //
        // Initialise the form
        //
        public Form1()
        {
            InitializeComponent();
        }



        //
        // Event hander for Browse to source PDF button
        //
        private void fileBrowse_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                txtPDFSource.Text = openFileDialog.FileName;
            }
        }



        //
        // Event handler for browse destination folder button
        //
        private void folderBrowse_Click(object sender, EventArgs e)
        {
            if (folderBrowserDialog.ShowDialog() == DialogResult.OK)
            {
                txtDestinationFolder.Text = folderBrowserDialog.SelectedPath;
            }
        }



        //
        // Event handler for extract images button
        //
        private void btnExtractImages_Click(object sender, EventArgs e)
        {
            String sourcePDF = "";
            String destFolder = "";
            MemoryStream memStream;
            byte[] bytes;

            sourcePDF = txtPDFSource.Text.Trim();
            destFolder = txtDestinationFolder.Text.Trim();

            // Validate we have a source and destination defined
            if (sourcePDF == "")
            {
                MessageBox.Show("Please choose a source PDF to extract images from.");
                fileBrowse_Click(this, null);
               return;
            }
            if ( destFolder == "")
            {
                MessageBox.Show("Please choose a destination folder to save extracted images to.");
                folderBrowse_Click(this, null);
                return;
            }

            // Ensure that this folder exists
            if (!Directory.Exists(destFolder))
                Directory.CreateDirectory(destFolder);

            //
            // If we are using Ghostwin to extract then do it here
            //
            if (chkUseGhostwin.Checked == true)
            {
                ExtractUsingGW(sourcePDF, destFolder);
                return;
            }


            // Imagecount is used to allow a unique key to be generated for each image - the key will be the imagecount itsself
            int imagecount = 0;

            //
            // Now we can start by loading the PDF document into our IText control
            //
            try
            {
                RandomAccessFileOrArray raf = new RandomAccessFileOrArray(sourcePDF);
                PdfReader rd = new PdfReader(raf,null);
                progressExtracting.Visible = true;
                progressExtracting.Maximum = rd.XrefSize;

                // Go through all of our objects
                for ( int i=0; i<rd.XrefSize; i++ )
                {
                    progressExtracting.Value = i + 1;

                    PdfObject pdfObj = rd.GetPdfObject(i);
                    if ( pdfObj != null )
                    {
                        if ( pdfObj.IsStream() == true )
                        {
                     
                            PdfStream stream = (PdfStream) pdfObj;

                            PdfObject subtype = stream.Get(PdfName.SUBTYPE);
                            if ( subtype != null )
                            {
                                if ( subtype.ToString() == PdfName.IMAGE.ToString() )
                                {
                                    // Now we need to get the image stream
                                    bytes = PdfReader.GetStreamBytesRaw((PRStream)stream);

                                    if (bytes != null )
                                    {
                                        PdfDictionary dic = (PdfDictionary)pdfObj;

                                        //
                                        // Get the information about this image
                                        //
                                        String filter = dic.Get(PdfName.FILTER).ToString();
                                        String width = dic.Get(PdfName.WIDTH).ToString();
                                        String height = dic.Get(PdfName.HEIGHT).ToString();
                                        String bpp = dic.Get(PdfName.BITSPERCOMPONENT).ToString();
                                        
                                        String colorspace = dic.Get(PdfName.COLORSPACE).ToString();

                                        try
                                        {
                                            BinaryWriter bw = null;
                                            FileStream fs = null;

                                            switch (filter)
                                            {
                                                case "/DCTDecode":
                                                    memStream = new MemoryStream(bytes);
                                                    memStream.Position = 0;

                                                    // Create the image from the stream
                                                    System.Drawing.Image img = System.Drawing.Image.FromStream(memStream);

                                                    // Save the image
                                                    img.Save(destFolder + "\\" + imagecount + ".jpg", ImageFormat.Jpeg);

                                                    break;


                                                case "/JBIG2Decode":

                                                    //                               STANDARD HEADER                                    NO PAGES, SEQUENTIAL ORDER
                                                    byte[] JBIGHeader = new byte[] { 0x97, 0x4a, 0x42, 0x32, 0x0d, 0x0a, 0x1a, 0x0a, 0x3 };
                                                    fs = new FileStream(destFolder + "\\" + imagecount + ".jb2", FileMode.Create, FileAccess.ReadWrite);
                                                    bw = new BinaryWriter(fs);
                                                    bw.Write(JBIGHeader);
                                                    bw.Write(bytes);

                                                    bw.Close();

                                                    break;

                                                case "/FlateDecode":

                                                    PixelFormat pf = PixelFormat.Format24bppRgb;
                                                    string fileext = ".png";
                                                    ImageFormat imgformat = ImageFormat.Png;

                                                    int actualw = Int32.Parse(width);
                                                    int actualh = Int32.Parse(height);

                                                    
                                                    switch (Int32.Parse(bpp))
                                                    {
                                                        case 1:
                                                            pf = PixelFormat.Format1bppIndexed;
                                                            imgformat = ImageFormat.Gif;
                                                            fileext = ".gif";
                                                            break;
                                                        case 8:
                                                            pf = PixelFormat.Format8bppIndexed;
                                                            imgformat = ImageFormat.Png;
                                                            fileext = ".png";
                                                            break;
                                                        case 24:
                                                            pf = PixelFormat.Format24bppRgb;
                                                            break;
                                                        default:
                                                            Debug.Write( "Unknown Pixel Format: " + bpp );
                                                            break;
                                                    }

                                                    // THE FOLLOWING CODE IS COMMENTED OUT IN THIS VERSION
                                                    //bytes = PdfReader.FlateDecode(bytes, true);

                                                    // Prepare a working image buffer to recreate the PDF image
                                                    //Bitmap bmp = new Bitmap(actualw, actualh, pf);
                                                    //BitmapData bmd = bmp.LockBits(new System.Drawing.Rectangle(0, 0, actualw, actualh), ImageLockMode.WriteOnly, pf);

                                                    //
                                                    // Now the complex part of working out how to process the byte stream
                                                    //
                                                    // We need the w,h,bpp, and component
                                                    // gray/rgb/cmyk
                                                    //
                                                    
                                                    //bmp.UnlockBits(bmd);
                                                    //bmp.Save(destFolder + "\\" + imagecount + fileext, imgformat);

                                                    break;

                                                default:

                                                    // Unhandled filter
                                                    break;
                                            }

                                            // Increase our image
                                            imagecount++;

                                        }
                                        catch
                                        {
                                            // Unknown image type
                                            Debug.Write("Image error");

                                            //MessageBox.Show("Unknown image or something!");
                                         
                                        }
                                    }
                                }
                            }
                        }
                    }
                }

            } catch {

                MessageBox.Show ("Error reading the PDF document." );
            }

          
            // Show how many we actually extracted
            MessageBox.Show("Processed " + imagecount + " images");
         
            // Should we show the output folder?
            ShowOutputFolder();
        }


        //
        // Extract using Ghostscript (renders full page versions of PDF)
        //
        public void ExtractUsingGW(String sourcePDF, String destFolder)
        {
            Process p = new Process();
            string output = "";
            try
            {

                p.StartInfo.UseShellExecute = false;
                p.StartInfo.RedirectStandardOutput = true;
                p.StartInfo.FileName = "gsconvert.bat";
                p.StartInfo.Arguments = "\"" + destFolder + "\" \"" + sourcePDF + "\"";
                p.Start();

                output = p.StandardOutput.ReadToEnd();

                p.WaitForExit();

            }
            catch
            {
                MessageBox.Show("Error running script. Please check discussions at www.fusion242.com for possible solution");
            }

            // Should we show the output folder?
            ShowOutputFolder();

            // Also open the output from the console process to view (for errors etc)
            frmViewOutput view = new frmViewOutput(output);
            view.Show();
            
        }



        //
        // Gets an image encoder
        //
        public static ImageCodecInfo GetImageEncoder(ImageFormat format)
        {
            return ImageCodecInfo.GetImageEncoders().ToList().Find(delegate(ImageCodecInfo codec)
            {
                return codec.FormatID == format.Guid;
            });
        }



        //
        // Show the output folder?
        //
        public void ShowOutputFolder()
        {
            // Did we check the box?
            if (chkOpenFolderAfterConversion.Checked == true)
            {
                Process.Start(txtDestinationFolder.Text);
            }
        }

        //
        // Event handler for Closing application
        //
        private void btnCloseApp_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }


       
    }

        
}
